/*
	$Id: Migrater.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg, Martin Svensson, Gregory Vincic
	Copyright (C) 2007 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.Application;
import net.sf.basedb.core.Config;
import net.sf.basedb.core.SessionControl;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
Migrates data from BASE 1 to BASE 2.

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/

public class Migrater extends Manager
{
	
	public static Set<String> supportedBase1Versions = new HashSet<String>();
	static
	{
		supportedBase1Versions.add("1.2.20");
		supportedBase1Versions.add("1.2.21");
	}
	
	/**
	Holds properties defined in migrate.properties
	 */
	protected Properties migrateProperties = null;
	//protected Properties baseProperties = null;
	/**
	BASE 1 connection, initiated by connect()
	 */
	Connection FROM = null;
	/**
	BASE 2 connection, initiated by connect()
	 */
	SessionControl TO = null;
	/**
	BASE 2 connection, used for impersonating users
	 */
	SessionControl sessionControl = null;
	/**
	BASE 2 connection, initiated by connect()
	 */
	Connection BASE2 = null;
	/**
	PreparedStatements
	 */
	//private HashMap<String, PreparedStatement> preparedStatements = new HashMap<String, PreparedStatement>();
	
	/**
	 The logger
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.Migrater");

	public void run()
	{
		try
		{
			ReporterTransfer reporterT = new ReporterTransfer(false);
			startTransfer(reporterT, "Reporter");
			RoleTransfer roleT = new RoleTransfer();
			startTransfer(roleT, "Role");
			QuotaTransfer quotaT = new QuotaTransfer();
			startTransfer(quotaT, "Quota");
			UserTransfer userT = new UserTransfer(roleT, quotaT);
			startTransfer(userT, "User");
			GroupTransfer groupT = new GroupTransfer(userT, quotaT);
			startTransfer(groupT, "Group");		
			PluginTransfer pluginT = new PluginTransfer(userT, groupT);
			startTransfer(pluginT, "Plugins");
			WizzzardTransfer wizzzardT = new WizzzardTransfer(userT, groupT);
			startTransfer(wizzzardT, "Wizzzard");
			ProtocolTypeTransfer protocolTypeT = new ProtocolTypeTransfer();
			startTransfer(protocolTypeT, "ProtocolType");
			NewsTransfer newsT = new NewsTransfer();
			startTransfer(newsT, "News");
			AnnotationTypeTransfer annotationTypeT = new AnnotationTypeTransfer(
					userT, groupT);
			startTransfer(annotationTypeT, "AnnotationType");
			PlateTypeTransfer plateTypeT = new PlateTypeTransfer(userT, groupT);
			startTransfer(plateTypeT, "PlateType");
			LabelTransfer labelT = new LabelTransfer(userT, groupT);
			startTransfer(labelT, "Label");
			HardwareTransfer hardwareT = new HardwareTransfer(userT, groupT);
			startTransfer(hardwareT, "Hardware");
			SoftwareTransfer softwareT = new SoftwareTransfer(userT, groupT);
			startTransfer(softwareT, "Software");
			UploadFileTransfer uploadFT = new UploadFileTransfer(userT, groupT,
					getProperty("userfiles"));
			startTransfer(uploadFT, "Upload");
			ProtocolTransfer protocolT = new ProtocolTransfer(userT, groupT,
					uploadFT, protocolTypeT);
			startTransfer(protocolT, "Protocol");
			SampleTissueTransfer sampleTissueT = new SampleTissueTransfer();
			startTransfer(sampleTissueT, "SampleTissue");
			SampleTransfer sampleT = new SampleTransfer(userT, groupT,
					protocolT, sampleTissueT);
			startTransfer(sampleT, "Sample");
			ExtractTransfer extractT = new ExtractTransfer(userT, groupT,
					protocolT, sampleT);
			startTransfer(extractT, "Extract");
			LabeledExtractTransfer labeledExtractT = new LabeledExtractTransfer(
					userT, groupT, protocolT, extractT, labelT);
			startTransfer(labeledExtractT, "LabeledExtract");
			AnnotationTransfer annotationT = new AnnotationTransfer(sampleT,
					annotationTypeT);
			startTransfer(annotationT, "Annotation");
			PlateEventTypeTransfer plateEventTypeT = new PlateEventTypeTransfer(
					protocolTypeT, plateTypeT);
			startTransfer(plateEventTypeT, "PlateEventType");
			UnmappedPlateTransfer unmappedPlateT = new UnmappedPlateTransfer(
					userT, groupT, plateTypeT, reporterT);
			startTransfer(unmappedPlateT, "UnmappedPlate");
			MappedPlateTransfer mappedPlateT = new MappedPlateTransfer(userT,
					groupT, plateTypeT, reporterT, unmappedPlateT);
			startTransfer(mappedPlateT, "MappedPlate");
			WellTransfer wellT = new WellTransfer(unmappedPlateT);
			startTransfer(wellT, "Well");
			PlateEventTransfer plateEventT = new PlateEventTransfer(
					mappedPlateT, plateEventTypeT, protocolT);
			startTransfer(plateEventT, "PlateEvent");
			ArrayTypeTransfer arrayTypeT = new ArrayTypeTransfer(userT, groupT,
					protocolT, uploadFT);
			startTransfer(arrayTypeT, "ArrayType");
			ArrayTypePlateTransfer arrayTypePlateT = new ArrayTypePlateTransfer(
					arrayTypeT, mappedPlateT);
			startTransfer(arrayTypePlateT, "ArrayTypePlate");
			ArrayBlockTransfer arrayBlockT = new ArrayBlockTransfer(arrayTypeT,
					mappedPlateT, reporterT);
			startTransfer(arrayBlockT, "ArrayBlock");
			ArrayBatchTransfer arrayBatchT = new ArrayBatchTransfer(userT,
					groupT, protocolT, arrayTypeT);
			startTransfer(arrayBatchT, "ArrayBatch");
			ArraySlideTransfer arraySlideT = new ArraySlideTransfer(userT,
					groupT, arrayBatchT);
			startTransfer(arraySlideT, "ArraySlide");
			HybridizationTransfer hybridizationT = new HybridizationTransfer(
					userT, groupT, arraySlideT, labeledExtractT, protocolT);
			startTransfer(hybridizationT, "Hybridization");
			ScanTransfer scanT = new ScanTransfer(userT, groupT,
					hybridizationT, hardwareT);
			startTransfer(scanT, "Scan");
			ImageFileTransfer imageFT = new ImageFileTransfer(userT, groupT,
					uploadFT, scanT, getProperty("userfiles") + "/rawimages");
			startTransfer(imageFT, "Image");
			RawBioAssayTransfer rawBioAssayT = new RawBioAssayTransfer(userT,
					groupT, scanT, arrayTypeT, uploadFT, softwareT, sampleT);
			startTransfer(rawBioAssayT, "RawBioAssay");
			createResumePoint(userT, groupT, rawBioAssayT);
			RawBioAssayDataTransfer rawBioAssayDataT = new RawBioAssayDataTransfer(
					rawBioAssayT, reporterT);
			startTransfer(rawBioAssayDataT, "RawBioAssayData");
			removeResumePoint();
			ExperimentTransfer experimentT = new ExperimentTransfer(userT,
					groupT, rawBioAssayT);
			startTransfer(experimentT, "Experiment");
			
			userT.finalizeQuota();

			print("END MIGRATION");
			log.info("Migration ended");
			Application.stop();
			System.exit(1);
		}
		catch (Throwable e)
		{
			log.error("The migration FAILED", e);
			stop();
		}
	}

	public void resume()
	{
		try
		{
			// Reporters must be mapped using external ID from BASE 1 and BASE 2
			ReporterTransfer reporterT = new ReporterTransfer(true);
			startTransfer(reporterT, "Reporter");
			
			// Pre-existing quota is loaded from BASE 2
			QuotaTransfer quotaT = new QuotaTransfer();
			startTransfer(quotaT, "Quota");

			ObjectInputStream ois = new ObjectInputStream(new FileInputStream("migration.tmp"));
			// Map of user ID:s are read from file
			UserTransfer userT = new UserTransfer(quotaT);
			userT.setup(FROM, TO, sessionControl, BASE2, migrateProperties);
			log.info("Loading users from 'migration.tmp' file...");
			userT.readRestorePointData(ois);
			log.info("Found " + userT.getIdMap().size() + " users");
			if (log.isDebugEnabled()) log.debug("User ID Map (Base 1 --> Base2): " + userT.getIdMap());

			// Map of group ID:s are read from file
			GroupTransfer groupT = new GroupTransfer();
			log.info("Loading groups from 'migration.tmp' file...");
			groupT.readRestorePointData(ois);
			log.info("Found " + groupT.getIdMap().size() + " groups");
			if (log.isDebugEnabled()) log.debug("Group ID Map (Base 1 --> Base2): " + groupT.getIdMap());
			
			// Map of raw bioassay ID:s are read from file
			RawBioAssayTransfer rawBioAssayT = new RawBioAssayTransfer();
			log.info("Loading raw bioassays from 'migration.tmp' file...");
			rawBioAssayT.readRestorePointData(ois);
			log.info("Found " + rawBioAssayT.getIdMap().size() + " raw bioassays");
			if (log.isDebugEnabled()) log.debug("Raw bioassay ID Map (Base 1 --> Base2): " + rawBioAssayT.getIdMap());
			ois.close();
			
			// RawBioassayTransfer runs normally, but checks if a raw bioassay already has raw data
			RawBioAssayDataTransfer rawBioAssayDataT = new RawBioAssayDataTransfer(rawBioAssayT, 
					reporterT);
			startTransfer(rawBioAssayDataT, "RawBioAssayData");
			removeResumePoint();
			
			// Experiment transfter runs normally
			ExperimentTransfer experimentT = new ExperimentTransfer(userT,
					groupT, rawBioAssayT);
			startTransfer(experimentT, "Experiment");
			
			// Runs normally
			userT.finalizeQuota();

			print("END MIGRATION");
			log.info("Migration ended");
			Application.stop();
			System.exit(1);
			
		}
		catch (Throwable e)
		{
			log.error("The migration FAILED", e);
			stop();
		}
	}
	
	private void createResumePoint(Transfer...transfers)
		throws IOException
	{
		log.info("Creating resume point");
		ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream("migration.tmp"));
		for (Transfer t: transfers)
		{
			log.info("Store to resume point: " + t);
			t.writeRestorePointData(oos);
		}
		oos.close();
	}
	
	private void removeResumePoint()
	{
		log.info("Removing resume point");
		java.io.File resumePoint = new java.io.File("migration.tmp");
		resumePoint.delete();
	}
	
	private void startTransfer(Transfer transfer, String property)
	{
		String configProperty = getProperty("include" + property);
		if (configProperty == null)
		{
			log.warn("include" + property + " is missing in migrate.properties");
		}
		else
		{
			if (configProperty.equals("yes"))
			{
				if (transfer.getDependencyStatus())
				{
					log.error("Skipping " + property, 
							new Throwable("Dependency failure in " + property + "Transfer"));
				}
				else
				{
					transfer.setPreparedStatements(getPreparedStatements());				
					transfer.setup(
							FROM, TO, sessionControl, BASE2, migrateProperties);
					transfer.start();
				}
			}
			else
			{
				log.warn("Skipping " + property + ", include" + property + "is set to no");
			}
		}
	}

	/**
	Creates connections to BASE 1 and BASE 2 Connection properties are found
	in the migrate.properties file which are loaded with configure().
	 */
	protected void connect()
	{
		// Connect to BASE 1
		log.info("Connecting to BASE1");
		String user = getProperty("db1.user");
		String password = getProperty("db1.password");
		String url = getProperty("db1.url") + "&autoReconnect=true";
		String driver = getProperty("db1.driver");
		try
		{
			Class.forName(driver);
			FROM = DriverManager.getConnection(url, user, password);
			if (getProperty("useLocalMySQL").equals("no"))
			{
				FROM.setReadOnly(true);
			}
		}
		catch (Exception e)
		{
			log.error("Failed when connecting to BASE 1. Please verify your settings", e);
			stop();
		}
		// Connect to BASE 2
		log.info("Connecting to BASE2");
		user = "root";
		password = getProperty("root.password");
		try
		{
			TO = Application.newSessionControl(null, null, null);
			TO.login(user, password, "MigrateSessionId", false);
			sessionControl = Application.newSessionControl(null, null, null);
			sessionControl.login(user, password, "impersonateSessionId", false);
		}
		catch (Exception e)
		{
			log.error("Failed when connecting to BASE 2. Please verify your settings", e);
			stop();
		}
		// Connect directly to BASE 2 database
		user = getProperty("db.username");
		password = getProperty("db.password");
		url = getProperty("db.url");
		driver = getProperty("db.driver");
		try
		{
			Class.forName(driver);
			BASE2 = DriverManager.getConnection(url, user, password);
			BASE2.setReadOnly(true);
		}
		catch (Exception e)
		{
			log.error("Failed when connecting directly to BASE 2 database. Please verify your settings", e);
			stop();
		}
	}

	/**
	Loads migration properties from migration.properties file.
	 */
	protected void configure()
	{
		log.info("Loading migration properties from file");
		String propertiesFile = "/migrate.properties";
		migrateProperties = new Properties();
		InputStream is = Migrater.class.getResourceAsStream(propertiesFile);
		if (is == null)
		{
			log.error("Couldn't find the configuration file.", 
					new IOException("Check that the " + propertiesFile 
							+ " file exists in your CLASSPATH"));
			stop();
		}
		else
		{
			try
			{
				migrateProperties.load(is);
				is.close();
			}
			catch (IOException e)
			{
				log.error("Couldn't load migration properties", e);
				stop();
			}
		}
		propertiesFile = "/base.config";
		/*
		baseProperties = new Properties();
		is = Migrater.class.getResourceAsStream(propertiesFile);
		if (is == null)
		{
			stop("configure()", new IOException("Check that the "
					+ propertiesFile + " file exists in your CLASSPATH"));
		}
		else
		{
			try
			{
				baseProperties.load(is);
				is.close();
			}
			catch (IOException e)
			{
				stop("configure()", e);
			}
		}
		*/
	}

	private void prepareStatementsFromFile()
	{
		log.info("Preparing sql statements from file");
		String qsFile = getProperty("sql.queries");
		String regexp = "^\\/\\*\\s+(\\w+)\\s+\\*\\/$";
		Pattern pattern = Pattern.compile(regexp);
		Map<String, PreparedStatement> preparedStatements = 
			new HashMap<String, PreparedStatement>();
		try
		{
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					Transfer.class.getResourceAsStream(qsFile)));
			if (reader == null)
			{
				log.error("Couldn't find file with sql statements", new IOException("Check that the " + qsFile
						+ " file exists in your CLASSPATH"));
				stop();
			}
			else
			{
				try
				{
					// Look up statement
					String line = reader.readLine();
					Matcher statementIdMatcher = pattern.matcher(line);
					StringBuilder sql = new StringBuilder();
					boolean append = false;
					String statementIdString = null;
					while (line != null)
					{
						if (append)
						{
							sql.append(line);
							sql.append('\n');
							sql.append(" "); // Incase we've forgotten to add
												// an extra space att the end of
												// a line
						}
						statementIdMatcher.reset(line);
						if (statementIdMatcher.matches())
						{
							append = true;
							sql = new StringBuilder();
							statementIdString = statementIdMatcher.group(1);
							log.debug("Preparing statementId '" + 	statementIdString + "'");
						}
						if (line.endsWith(";"))
						{
							append = false;
							String query = sql.toString().replaceAll(";", "");
							preparedStatements.put(
									statementIdString,
									FROM.prepareStatement(query));
						}
						line = reader.readLine();
					}
					reader.close();
				}
				catch (IOException e)
				{
					log.error("Failed reading the file 'sql.queries'", e);
					stop();
				}
			}
		}
		catch (Exception e)
		{
			log.error("Couldn't prepare statements from file", e);
			stop();
		}
		setPreparedStatements(preparedStatements);
	}
	
	protected String getProperty(String key)
	{
		String value = migrateProperties.getProperty(key);
		if (value == null)
		{
			value = Config.getString(key);
		}
		return value;
	}

	private void verifySchemaVersion()
	{
		try
		{
			log.info("Verifying schema version of BASE1");
			PreparedStatement ps = prepareStatementFromFile("verifySchemaVersion");
			ResultSet rs = ps.executeQuery();	
			rs.next();
			String schemaVersion = rs.getString(2);
			ps.close();
			if (!supportedBase1Versions.contains(schemaVersion))
			{
				log.error("Migration cannot proceed due to incompatible schema version. Your schema version is "
						+ schemaVersion + " and this migration needs one of: " + supportedBase1Versions);
				stop();
			}			
		}
		catch (SQLException e)
		{
			log.error("Couldn't get BASE1's schema version", e);
			stop();
		}
	}

	/**
	 
	 */
	public static void main(String[] args)
	{
		System.err.println("-----------------------------------------------------------------------");
		System.err.println("Disclaimer");
		System.err.println("We have made extensive testing of the migration from BASE1.2 to BASE2");
		System.err.println("To our knowledge the migration works but we cannot guarantee perfect");
		System.err.println("functionality. The migration tool is supplied as is and usage is done");
		System.err.println("at your risk. When you start the migration tool you are required to");
		System.err.println("pass parameter 'disclaimer_understood' to the migrate_from_1.2.sh");
		System.err.println("script. Remember to check that the migration performed well before you");
		System.err.println("decide to delete your BASE1.2 installation.");
		System.err.println("-----------------------------------------------------------------------");

		java.io.File restorePoint = new java.io.File("migration.tmp");
		boolean resumeMigration = false;

		if (args == null || args.length == 0 || !"disclaimer_understood".equals(args[0]))
		{
			System.err.println("Use: \"./migrate_from_1.2.sh disclaimer_understood\" to start the program");
			System.exit(0);
		}
		else if (restorePoint.exists())
		{
			System.err.println("FOUND DATA FROM A PREVIOUSLY INTERRUPTED MIGRATION.");
			System.err.println("\n   Press ENTER to resume the migration\nor\n   Ctrl-C to abort\n");
			System.err.println("To start a new migration delete the './migration.tmp' file");
			System.err.println("before starting the migration program.");
			waitForEnter();
			resumeMigration = true;
		}
		else
		{
			System.err.println("Migration switch found, press ENTER to continue with migration... (Ctrl-C to stop)");
			waitForEnter();
		}
		
		System.err.println(resumeMigration ? "Resuming migration" : "Starting migration...");
		Application.start(false);
		Migrater migrater = new Migrater();
		migrater.configure();
		migrater.connect();
		migrater.prepareStatementsFromFile();
		migrater.verifySchemaVersion();
		log.info("Migration initialisation DONE");
		if (resumeMigration)
		{
			migrater.resume();
		}
		else
		{
			migrater.run();
		}
	}
	
	public static void waitForEnter()
	{
		try
		{
			System.in.read();
			System.in.skip(System.in.available());
		}
		catch (Exception ex)
		{}
	}
		
}
