/**
	$Id: SpotImageFormatter.java 4838 2009-03-24 12:10:48Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.formatter;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataUtil;
import net.sf.basedb.core.data.RawData;
import net.sf.basedb.util.formatter.Formatter;

/**

	@author nicklas
	@version 2.0
	@base.modified $Date: 2009-03-24 13:10:48 +0100 (Tue, 24 Mar 2009) $
*/
public class SpotImageFormatter
	implements Formatter<Integer>
{

	private final DbControl dc;
	private final RawDataType rawDataType;
	private final String ID;
	private final String root;
	public String imgClass = "spotimage";
	
	public SpotImageFormatter(DbControl dc, RawDataType rawDataType, String root)
	{
		this.dc = dc;
		this.rawDataType = rawDataType;
		this.root = root;
		this.ID = dc.getSessionControl().getId();
	}
	
	/**
		Sets the class attribute for the generated &lt;img&gt; tag
		that displays the spot image. The default value
		is "spotimage".
		@since 2.11
	*/
	public void setImgClass(String imgClass)
	{
		this.imgClass = imgClass;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	public String format(Integer value)
	{
		RawData rawData = rawDataType.getRawDataById(dc, value);
		RawBioAssay rawBioAssay = RawDataUtil.getRawBioAssay(dc, rawData);
		
		StringBuilder sb = new StringBuilder();
		if (rawBioAssay.hasSpotImages())
		{
			sb.append("<img");
			if (imgClass != null) sb.append(" class=\"").append(imgClass).append("\"");
			sb.append(" src=\"").append(root).append("/views/rawbioassays/rawdata/spotimage/");
			sb.append(ID).append("/").append(rawBioAssay.getId()).append("/");
			sb.append(rawData.getId()).append("/spot").append(rawData.getPosition()).append(".png");
			sb.append("\">");
		}
		else
		{
			sb.append("<i>- none -</i>");
		}
		return sb.toString();
	}
	/**
		Not supported.
		@since 2.2
	*/
	public Integer parseString(String value)
	{
		throw new UnsupportedOperationException("parseString");
	}
	// -------------------------------------------

}
