/**
	$Id $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.snapshot.SnapshotManager;
import net.sf.basedb.util.BioAssaySetUtil;

/**	
	Data loader that gets annotation values for a bioassay including
	those that are present on the parent raw bioassay(s) without being
	directly inherited by the bioassay. See 
	{@link BioAssaySetUtil#getAnnotationValues(DbControl, SnapshotManager, BioAssay, net.sf.basedb.core.AnnotationType)}
	
	@author Nicklas
	@version 2.13
	@base.modified $Date: 2009-10-13 12:44:33 +0200 (Tue, 13 Oct 2009) $
*/
public class BioAssayExperimentalFactorLoader 
	implements DataLoader<BioAssay>
{

	private final SnapshotManager snapshotManager;
	public BioAssayExperimentalFactorLoader()
	{
		this.snapshotManager = new SnapshotManager();
	}
	
	/**
		Creates an experimental factor loader that uses a specific snapshot manager
		for loading the annotation values.
		@since 2.14
	*/
	public BioAssayExperimentalFactorLoader(SnapshotManager snapshotManager)
	{
		this.snapshotManager = snapshotManager;
	}
	
	@Override
	public Object getData(ExportedProperty exportedProperty, BioAssay item) 
	{
		DbControl dc = item.getDbControl();
		return BioAssaySetUtil.getAnnotationValues(dc, snapshotManager, item, exportedProperty.annotationType);
	}

}
