/**
	$Id: ExportedProperty.java 5307 2010-04-12 16:23:00Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Metadata;
import net.sf.basedb.core.Unit;
import net.sf.basedb.core.Metadata.PropertyPath;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.formatter.Formatter;

/**

 @author nicklas
 @version 2.2
 @base.modified $Date: 2010-04-12 18:23:00 +0200 (Mon, 12 Apr 2010) $
 */
public class ExportedProperty
{
	public final String name;
	public final String title;
	public final AnnotationType annotationType;
	public final Unit unit;
	public final Formatter formatter;
	public final DataLoader dataloader;
	public final PropertyPath propertyPath;

	public static ExportedProperty createExportedProperty(String path, String title, Metadata metadata, Formatter formatter)
	{
		PropertyPath propertyPath = metadata.getPropertyPath(path, false);
		return new ExportedProperty(path, title, propertyPath, null, null, formatter, null);
	}

	
	/**
	 	@since 2.15
	*/
	public static ExportedProperty parse(DbControl dc, Metadata metadata, String column, String columnPrefix, 
			boolean annotatable, Formatter formatter, boolean sameUnits, DataLoader dataloader)
	{
		String[] p = column.split(":");
		String name = p[0];
		String title = p.length > 1 ? p[1] : p[0];
		AnnotationType at = null;
		Unit unit = null;
		String columnSuffix = null;
		PropertyPath propertyPath = null;
		if (name.startsWith("#") && annotatable) // && itemType != Item.SPOTDATA)
		{
			// Annotation: properties[i] = #annotationTypeId
			int annotationTypeId = Values.getInt(name.substring(1));
			at = AnnotationType.getById(dc, annotationTypeId);
			name = "annotationtype_"+annotationTypeId;
			if (sameUnits && at.supportUnits()) 
			{
				unit = at.getDefaultUnit();
				if (unit != null) columnSuffix = " (" + unit.getDisplaySymbol() + ")";
			}
		}
		else if (dataloader == null)
		{
			propertyPath = metadata.getPropertyPath(name, false);
		}
		if (columnPrefix != null) title = columnPrefix + title;
		if (columnSuffix != null) title += columnSuffix;
		return new ExportedProperty(name, title, propertyPath, at, unit, formatter, dataloader);
	}

	/**
		@since 2.15
	*/
	public ExportedProperty(String name, String title, PropertyPath propertyPath,
			AnnotationType annotationType, Unit unit, Formatter formatter, DataLoader dataloader)
	{
		this.name = name;
		this.title = title;
		this.propertyPath = propertyPath;
		this.annotationType = annotationType;
		this.formatter = formatter;
		this.unit = unit;
		this.dataloader = dataloader;
	}
}
