/**
	$Id: ParentBioMaterialLoader.java 5299 2010-03-31 11:46:34Z nicklas $

	Copyright (C) 2009 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.MeasuredBioMaterial;
import net.sf.basedb.core.Metadata;

/**	
	This class get data from the parents to a {@link MeasuredBioMaterial}
	@author martin
	@version 2.12
	@base.modified $Date: 2010-03-31 13:46:34 +0200 (Wed, 31 Mar 2010) $
*/

public class ParentBioMaterialLoader 
	implements DataLoader<MeasuredBioMaterial<?>>
{

	public ParentBioMaterialLoader()
	{}
	
	@SuppressWarnings("unchecked")
	@Override
	public Object getData(ExportedProperty exportedProperty, MeasuredBioMaterial<?> item) 
		throws Exception
	{
		Object parents = null;
		DbControl dc = item.getDbControl();
		if (item.isPooled())
		{
			ItemQuery<? extends MeasuredBioMaterial> pooledQuery = item.getCreationEvent().getSources();
			pooledQuery.include(Include.ALL);
			parents = pooledQuery.list(dc);
		}
		else
		{
			Metadata metadata = Metadata.getInstance(item.getType().getDataClass());
			try
			{
				parents = exportedProperty.propertyPath.getValue(dc, item);
			}
			catch(Throwable e)
			{}
		}
		return parents;
	}

}
