/**
	$Id: Form.java 5111 2009-09-30 10:56:51Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.table;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;


/**
	An alternative layout for tables which contains only one row.
	The table and columns should be defined as usual. Values for 
	the columns are also set with the &lt;tbl:cell&gt; tag as asual.
	<p>
	This layout shows the "row" as a pair of left-right-oriented 
	header/value pair. The default width is 1 which means that
	the header/value are stacked vertically. The width can be
	increased to get several "columns" of header/value pairs.
	<p>
	This layout was developed because we needed to support a form-based
	layout were the order and visibility of the header/values 
	can be modified by the user. With this layout the same configuration
	dialog that is used for normal tables can be used.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;tbl:form
   	width=...
   &gt;
   	&lt;tbl:cell&gt;...&lt;/tbl:cell&gt;
   &lt;/tbl:form&gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>width</td>
		<td>1</td>
		<td>no</td>
		<td>
			The number of columns (header/value) pairs in the form.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.14
	@base.modified $Date: 2009-09-30 12:56:51 +0200 (Wed, 30 Sep 2009) $
*/
public class Form
	extends TagSupport
{

	private static final long serialVersionUID = -6667213780186530313L;
	private Table table;
	private Map<String, String> headers;
	
	private int width = 1;
	
	public void setWidth(int width)
	{
		this.width = width;
	}
	
	@Override
	public int doStartTag() 
		throws JspException
	{
		table = (Table)findAncestorWithClass(this, Table.class);
		// Make a copy of the headers
		headers = new HashMap<String, String>(table.getColumnContent());
		table.getColumnContent().clear();
		return EVAL_BODY_INCLUDE;
	}

	
	@Override
	public int doEndTag() 
		throws JspException
	{
		// Get the contents
		Map<String, String> contents = table.getColumnContent();
		StringBuilder sb = new StringBuilder();
		int colNum = 0;
		for (String col : table.getVisibleColumns())
		{
			String header = headers.get(col);
			if (header == null) header = "<td>&nbsp;</td>";
			String content = contents.get(col);
			if (content == null) content = "<td>&nbsp;</td>";
			if (colNum % width == 0) sb.append("<tr>");
			sb.append(header);
			sb.append(content);
			++colNum;
			if (colNum % width == 0) sb.append("</tr>");
		}
		if (colNum % width != 0) sb.append("</tr>");
		contents.clear();
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}


	
	
}
