/*
 $Id: ScanData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;

/**
	This class holds information about an scan.
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.Scan
	@see <a href="../../../../../../html/developerdoc/api/data_api.rawdata.html">Hybridzations and raw data overview</a>
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
	@hibernate.class table="`Scans`" lazy="false"
*/
public class ScanData 
	extends AnnotatedData
	implements RegisteredData
{

	public ScanData()
	{}

	/*
		From the RegisteredData interface
		-------------------------------------------
	*/
	private Date entryDate;
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -------------------------------------------

	private HybridizationData hybridization;
	/**
		Get the hybridization.
		@hibernate.many-to-one column="`hybridization_id`" not-null="true" outer-join="false"
	*/
	public HybridizationData getHybridization()
	{
		return hybridization;
	}
	public void setHybridization(HybridizationData hybridization)
	{
		this.hybridization = hybridization;
	}
	
	private HardwareData scanner;
	/**
		Get the hardware, ie. scanner, used to scan the images.
		@hibernate.many-to-one column="`hardware_id`" not-null="false" outer-join="false"
	*/
	public HardwareData getScanner()
	{
		return scanner;
	}
	public void setScanner(HardwareData scanner)
	{
		this.scanner = scanner;
	}

	private ProtocolData protocol;
	/**
		Get the protocol used while scanning.
		@hibernate.many-to-one column="`protocol_id`" not-null="false" outer-join="false"
	*/
	public ProtocolData getProtocol()
	{
		return protocol;
	}
	public void setProtocol(ProtocolData protocol)
	{
		this.protocol = protocol;
	}

	private Set<ImageData> images;
	/**
		This is the inverse end.
		@see ImageData#getScan()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`scan_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ImageData"
	*/
	Set<ImageData> getImages()
	{
		return images;
	}
	void setImages(Set<ImageData> images)
	{
		this.images = images;
	}

	private Set<RawBioAssayData> rawBioAssays;
	/**
		This is the inverse end.
		@see RawBioAssayData#getScan()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`scan_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.RawBioAssayData"
	*/
	Set<RawBioAssayData> getRawBioAssays()
	{
		return rawBioAssays;
	}
	void setRawBioAssays(Set<RawBioAssayData> rawBioAssays)
	{
		this.rawBioAssays = rawBioAssays;
	}


}
