/*
	$Id:AbstractDataFileHandler.java 3820 2007-10-12 10:03:18Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.filehandler;

import java.util.HashMap;
import java.util.Map;

import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.FileStoreEnabled;

/**
	This abstract class should, if possible, be used as a
	super-class for all classes implementing the 
	{@link DataFileValidator} interface. We consider this interface
	to be part of the public API only on the client side, not on the
	implementor side. This means that we may add additional methods
	(but not change existing ones) to the interface at any time.
	All new methods will get default implementations in this abstract 
	class to not break existing implementions.
	
	@author Nicklas
	@version 2.5
	@base.modified $Date:2007-10-12 12:03:18 +0200 (Fri, 12 Oct 2007) $
*/
public abstract class AbstractDataFileHandler
	implements DataFileHandler
{
	private FileStoreEnabled item;
	private Map<String, FileSetMember> members;
	
	protected AbstractDataFileHandler()
	{}
	
	/*
		From the DataFileHandler interface
		----------------------------------------
	*/
	public void setItem(FileStoreEnabled item)
	{
		this.item = item;
	}
	public void setFile(FileSetMember member)
	{
		if (members == null)
		{
			members = new HashMap<String, FileSetMember>();
		}
		members.put(member.getDataFileType().getExternalId(), member);
	}
	// -------------------------------------------

	/**
		Get the itemt that was set by {@link #setItem(FileStoreEnabled)}
	*/
	protected FileStoreEnabled getItem()
	{
		return item;
	}
	
	/**
		Get the file set member of a specific {@link DataFileType}.
		@param dataFileType The external ID of the data file type
		@return The file set member or null if no member of
			that type is found
	 */
	protected FileSetMember getMember(String dataFileType)
	{
		return members == null ? null : members.get(dataFileType);
	}
	
}
