/**
	$Id: AnnotationSetLogger.java 5068 2009-08-20 07:14:03Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.log.db;

import org.hibernate.Query;

import net.sf.basedb.core.Item;
import net.sf.basedb.core.LogControl;
import net.sf.basedb.core.PredefinedQuery;
import net.sf.basedb.core.data.AnnotationSetData;
import net.sf.basedb.core.data.ChangeHistoryDetailData;
import net.sf.basedb.core.data.LoggableData;
import net.sf.basedb.core.log.EntityDetails;
import net.sf.basedb.core.log.ChangeType;
import net.sf.basedb.core.log.EntityLogger;
import net.sf.basedb.core.log.LogManager;

/**
	Logger for annotations sets that detects and logs
	changes to inherited anntations as updates to the parent 
	item that are inheriting the annotations.
	
	@author Nicklas
	@version 2.13
	@base.modified $Date: 2009-08-20 09:14:03 +0200 (Thu, 20 Aug 2009) $
*/
public class AnnotationSetLogger
	implements EntityLogger
{

	private final boolean detailedProperties;
	
	public AnnotationSetLogger(boolean detailedProperties)
	{
		this.detailedProperties = detailedProperties;
	}
	
	/*
		From the EntityLogger interface
		-------------------------------
	*/
	@Override
	public void logChanges(LogManager logManager, EntityDetails details)
	{
		// We are only logging update events; CREATE and DELETE events are
		// usually also catched by other loggers
		if (details.getChangeType() != ChangeType.UPDATE) return;
		
		LogControl logControl = logManager.getLogControl();
		AnnotationSetData as = (AnnotationSetData)details.getEntity();
		
		// Get the type of the parent item
		Integer itemType = as.getItemType();
		
		Class parentClass = Item.fromValue(itemType).getDataClass();
		// If the parent item isn't implementing the LoggableData interface
		// we don't log the changes
		if (!LoggableData.class.isAssignableFrom(parentClass)) return;
		
		// Get the id of the parent item
		Query q = logControl.createHqlQuery(
			PredefinedQuery.getQueryString("DBLOG_GET_ITEMID_WITH_ANNOTATION", parentClass.getName()));
		q.setInteger("annotationSetId", as.getId());
		Integer parentId = (Integer)q.uniqueResult();
		
		// May happen in case the parent item has been deleted in this transaction
		if (parentId == null) return;
		
		ChangeHistoryDetailData change = new ChangeHistoryDetailData();
		change.setChangeType(ChangeType.UPDATE.getValue());
		change.setItemId(parentId);
		change.setItemType(itemType);
		if (detailedProperties)
		{
			change.setChangeInfo(details.getModifiedProperties("Updated: ", ", ", null));
		}
		((DbLogManager)logManager).logChangeDetails(change);
	}
	// -------------------------------
}
