/**
	$Id: ForwardingSignalHandler.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.signal;

/**
	A signal handler implementation that forwards the signal to
	another signal transporter.

	@author nicklas
	@version 2.6
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public class ForwardingSignalHandler
	extends AbstractSignalHandler
{

	/**
		Log signals processing.
	*/
	private static final org.apache.log4j.Logger logger = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.signal.ForwardingSignalHandler");

	/**
		The signal transporter to forward signals to.
	*/
	private final SignalTransporter forwardTo;
	
	/**
		Create a new forwarding signal handler.
		@param forwardTo The signal transporter to forward signals to
	*/
	public ForwardingSignalHandler(SignalTransporter forwardTo)
	{
		super();
		this.forwardTo = forwardTo;
	}
	
	/*
		From the SignalHandler interface
		-------------------------------------------
	*/
	/**
		Forwards the signal to the signal transporter given in the 
		constructor. This method doesn't check if the signal is supported or
		not.
	*/
	public void handleSignal(Signal signal)
	{
		logger.info("Forwarding signal: " + signal);
		forwardTo.send(signal);
	}
	// -------------------------------------------

}
