/*
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

/**
	Helper class for implementing equals and hash code calculations in
	classes.
	@author nicklas
	@since 2.15
*/
public class EqualsHelper
{

	/**
		Check if two objects are equal according to their equals
		implementation. This method is safe to use with null values
		and returns true if both objects are null. Otherwise,
		it calls o1.equals(o2).
		
		@param o1 The first object
		@param o2 The second object
		@return TRUE if the objects are equal
	*/
	public static boolean equals(Object o1, Object o2)
	{
		return o1 == o2 || (o1 != null && o1.equals(o2));
	}
	
	/**
		Get the hashcode for an object. This method is safe to use
		with null and returns 0.
		@param o The object to get the hash code for
		@return The hash code
	*/
	public static int hashCode(Object o)
	{
		return o == null ? 0 : o.hashCode();
	}
	
}
