/*
	$Id: ToStringComparator.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
 
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

 	You should have received a copy of the GNU General Public License
 	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.util.Comparator;

/**
	An implementation of the <code>Comparator</code> interface which uses
	the result of the <code>toString</code> method to compare the objects.
	Ie. by using this comparator objects are sorted according to the 
	<code>toString()</code>
	values.
	<p>
	Note: this comparator may impose orderings that are inconsistent with equals.
	
	@author Nicklas
	@version 2.0
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $

*/
public class ToStringComparator<T>
	implements Comparator<T>
{

	private final boolean descending;
	
	/**
		Create a new comparator.
		@param descending TRUE to sort objects in descending order (z--a), 
			FALSE to sort the objects in ascending order (a--z)
	*/
	public ToStringComparator(boolean descending)
	{
		this.descending = descending;
	}

	public int compare(T o1, T o2)
	{
		return descending ? o2.toString().compareTo(o1.toString()) : o1.toString().compareTo(o2.toString());
	}

}
