/**
	$Id: RemoveAllTransformer.java 4712 2008-12-19 13:55:13Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.collections;

import java.util.Collection;

/**
	Collection transformer implementation that removes all elements in
	the source collection from the destination collection. This
	is equivalent to <code>destination.removeAll(source)</code>.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2008-12-19 14:55:13 +0100 (Fri, 19 Dec 2008) $
*/
public class RemoveAllTransformer<I>
	implements CollectionTransformer<I, I>
{

	public RemoveAllTransformer()
	{}
	/*
		From the CollectionTransformer interface
		----------------------------------------
	*/
	/**
		Remove all elements in the source collection from the destination collection.
		@return The number of elements removed
	*/
	@Override
	public int transform(Collection<? extends I> source, Collection<? super I> destination)
	{
		int size = destination.size();
		destination.removeAll(source);
		return size - destination.size();
	}
	// ------------------------------------------

}
