/**
	$Id: DynamicField.java 5258 2010-02-26 11:05:49Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.export.spotdata;

import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DynamicQuery;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.util.formatter.Formatter;

/**
	Represents a spot or reporter field that is going to be exported
	from a bioassay set. An implementation needs to provide the following
	information and services:
	<ul>
	<li>An {@link Expression} that is added to the selection list of a
		{@link DynamicQuery}.
	<li>A {@link Formatter} that is used to format the values that are
		selected by the expression before they are written to the file
	<li>A title that is used as a column header for the data
	</ul>

	@author Nicklas
	@version 2.12
	@base.modified $Date: 2010-02-26 12:05:49 +0100 (Fri, 26 Feb 2010) $
*/
public interface DynamicField
	extends ExportableField
{
	/**
		Get an expression that selects the data that should be exported.
		
		@param dc The DbControl that is used for database access
		@param query The query the expression will be used with
		@param source The source bioassay set
		@param forAverage TRUE if the expression should generate an
			average of multiple values
		@return An expression, or null if it is not possible to create an
			expression in the given case (this will normally be replaced with 
			an empty column in the output)
	*/
	public Expression getExpression(DbControl dc, DynamicQuery query, BioAssaySet source, boolean forAverage);

}
