/*
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import java.io.InputStream;
import java.io.OutputStream;

/**
	A simple program that copies what is sent on standard input to standard output.
	In case of an error the stacktrace is printed to the standard error stream.
	It accepts one parameter on the command line:
	error: Simulate an error (exit code = 1)
	slow: Simulate a long-running program by waiting 
		* 10 seconds before copying,
		* 1 second for each kb of input
		* 10 seconds at the end
	@author nicklas
	@since 2.15
*/
public class CopyStdin2Stdout 
{

	public static void main(String[] args) 
	{
		String cmd = args.length > 0 ? args[0] : null;
		boolean simulateError = "error".equals(cmd);
		boolean isSlow = "slow".equals(cmd);
		byte[] buffer = new byte[1024];	
		InputStream in = System.in;
		OutputStream out = System.out;
		int bytes = 0;
		try
		{
			if (isSlow) Thread.sleep(10000);
			if (simulateError) throw new RuntimeException("Simulating an error");
			while (bytes != -1) // -1 = end of stream
			{
				bytes = in.read(buffer, 0, buffer.length);
				if (bytes > 0)
				{
					out.write(buffer, 0, bytes);
				}
				if (isSlow) Thread.sleep(1000);
			}
			if (isSlow) Thread.sleep(10000);
		}
		catch (Throwable t)
		{
			t.printStackTrace();
			System.exit(1);
		}
	}

}
