/*
	$Id: TestFormula.java 5293 2010-03-30 13:40:33Z nicklas $

	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Arrays;
import java.util.Date;

public class TestFormula
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing formulas");
		write_header();
		// Standard tests: create, load, list
		int id = test_create("add", Formula.Type.COLUMN_EXPRESSION, 
			"genepix",  new String[] { "ch(1) + ch(2)" }, true);
		int id2 = test_create("raw fg mean", Formula.Type.INTENSITY_EXPRESSION,
			"genepix", new String[] { "raw('ch1FgMean')", "raw('ch2FgMean')" }, false);
		test_load(id);
		test_list(-1);

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		test_delete(id2);
		write("++Testing formulas "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String name, Formula.Type formulaType, String rawDataType, String[] formulas, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.FORMULA)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Formula f = Formula.getNew(dc);
			if (setAll)
			{
				f.setName("Test formula");
				f.setDescription("Added at "+new Date());
				f.setChannels(2);
				f.setParser(Formula.Parser.JEP);
				Coloring c = f.getColoring();
				c.setLogarithmic(true);
				c.setMidValue(0f);
				c.setMinValue(-10f);
				c.setMaxValue(10f);
				c.setUsingColors(true);
			}
			if (name != null) f.setName(name);
			f.setFormulaType(formulaType);
			f.setRawDataType(RawDataTypes.getRawDataType(rawDataType));
			f.setFormulas(Arrays.asList(formulas));
			dc.saveItem(f);
			dc.commit();
			id = f.getId();
			write_item(0, f);
			write("--Create formula OK");
		}
		catch (Throwable ex)
		{
			write("--Create formula FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Formula f = Formula.getById(dc, id);
			write_item(0, f);
			write("--Load formula OK");
		}
		catch (Throwable ex)
		{
			write("--Load formula FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Formula> l = Formula.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List formulas OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List formulas FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Formula f = Formula.getById(dc, id);
			dc.deleteItem(f);
			dc.commit();
			write("--Delete formula OK");
		}
		catch (Throwable ex)
		{
			write("--Delete formula FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tType\tParser\tRaw data type\tChannels\tFormulas");
			write("-- \t-- \t--------- \t-----------\t----\t------\t-------------\t--------\t--------");
		}
	}
	static void write_item(int i, Formula f)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+f.getId()+"\t"+f.getName()+"\t"+f.getDescription()+"\t"+
				f.getFormulaType()+"\t"+f.getParser()+"\t"+f.getRawDataType()+"\t"+f.getChannels()+"\t"+
				f.getFormulas()
			);
		}
	}
	static void write(String message)
	{
		System.out.println(message);
	}
}
