/*
	$Id: TestItemKey.java 5024 2009-07-27 11:54:33Z nicklas $

	Copyright (C) 2005, 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import java.util.EnumSet;

public class TestItemKey
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing item key");
		write_header();
		
		// Delete "garbage" from previous tests
		ItemKey.deleteUnusedItemKeys();

		// Extra test: create user and group for the key
		int userId = TestUser.test_create(true);
		int groupId = TestGroup.test_create(true);
		
		// Standard tests: create, load
		int id = test_create(userId, EnumSet.of(Permission.WRITE), groupId, EnumSet.of(Permission.READ));
		test_load(id);

		// Extra tests: create new key with same permission -> id2 == id
		int id2 = test_create(userId, EnumSet.of(Permission.WRITE), groupId, EnumSet.of(Permission.READ));
		write("--id==id2 "+(id==id2 ? "OK" : "FAILED id="+id+"; id2="+id2));
		ok = ok && (id == id2);

		// Extra tests: create new key with different permission -> id3 != id
		int id3 = test_create(userId, EnumSet.of(Permission.USE), groupId, EnumSet.of(Permission.READ));
		write("--id!=id3 "+(id!=id3 ? "OK" : "FAILED id="+id+"; id3="+id3));
		ok = ok && (id != id3);


		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestUser.test_delete(userId);
		TestGroup.test_delete(groupId);

		test_delete_unused(2);

		write("++Testing item key "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int userId, EnumSet<Permission> userPermissions, int groupId, EnumSet<Permission> groupPermissions)
	{
		if (userId == 0 || groupId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, userId);
			UserPermissions up = new UserPermissions();
			up.setPermissions(u, userPermissions);
			Group g = Group.getById(dc, groupId);
			GroupPermissions gp = new GroupPermissions();
			gp.setPermissions(g, groupPermissions);
			ItemKey ik = ItemKey.getNewOrExisting(dc, up, gp);
			id = ik.getId();
			write_item(0, ik);
			write("--Create item key OK");
		}
		catch (Throwable ex)
		{
			write("--Create item key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemKey ik = ItemKey.getById(dc, id);
			write_item(0, ik);
			write("--Load item key OK");
		}
		catch (Throwable ex)
		{
			write("--Load item key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete_unused(int expected)
	{
		try
		{
			int numDeleted = ItemKey.deleteUnusedItemKeys();
			if (expected >= 0 && numDeleted != expected)
			{
				throw new BaseException("Unexpected number of unused item keys: " + 
					numDeleted + "; expected " + expected);
			}
			write("--Delete unused item keys OK (" + expected + ")");
		}
		catch (Throwable ex)
		{
			write("--Delete unused item keys FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}


	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID");
			write("-- \t--");
		}
	}
	static void write_item(int i, ItemKey ik)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+ik.getId());
	}
	static void write(String message)
	{
		System.out.println(message);
	}


}

