/*
	$Id: TestNews.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import java.util.Date;

public class TestNews
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing news");
		write_header();
		// Standard tests: create, load, list
		int id = test_create(true);
		int id2 = test_create(false);
		test_load(id);
		test_list(-1);

		// Extra tests:
		test_list_nologin();

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		test_delete(id2);
		write("++Testing news "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.NEWS)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Date yesterday = new Date(System.currentTimeMillis()-1000*60*60*24); 
			Date today = new Date();
			Date tomorrow = new Date(System.currentTimeMillis()+1000*60*60*24);
			News n = News.getNew(dc, yesterday, today);
			if (setAll)
			{
				n.setName("Test news");
				n.setDescription("Added at "+new Date());
				n.setEndDate(tomorrow);
			}
			dc.saveItem(n);
			dc.commit();
			id = n.getId();
			write_item(0, n);
			write("--Create news OK");
		}
		catch (Throwable ex)
		{
			write("--Create news FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			News n = News.getById(dc, id);
			write_item(0, n);
			write("--Load news OK");
		}
		catch (Throwable ex)
		{
			write("--Load news FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<News> l = News.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List news OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List news FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			News n = News.getById(dc, id);
			dc.deleteItem(n);
			dc.commit();
			write("--Delete news OK");
		}
		catch (Throwable ex)
		{
			write("--Delete news FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tStart date\tNews date\tEnd date");
			write("-- \t-- \t--------- \t-----------\t----------\t---------\t--------");
		}
	}
	static void write_item(int i, News n)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+n.getId()+"\t"+n.getName()+"\t"+n.getDescription()+
			"\t"+n.getStartDate()+"\t"+n.getNewsDate()+"\t"+n.getEndDate());
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_list_nologin()
	{
		DbControl dc = null;
		try
		{
			TestUtil.logout();
			dc = TestUtil.getDbControl();
			ItemResultList<News> l = News.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List news no login OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List news no login FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
			TestUtil.login();
		}
	}




}

