/*
	$Id: TestPluginType.java 5060 2009-08-19 07:02:11Z nicklas $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;
import java.util.Date;

public class TestPluginType
{

	static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing plugin type");
		write_header();
		// Standard tests: load and list
		int id = test_create("NullPluginInterface", null, true);
		int id2 = test_create("JarPluginInterface", "JarPlugin.jar", false);
		test_load(id);
		test_list();
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		test_delete(id2);

		write("++Testing plugin type "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String interfaceName, String jarPath, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.PLUGINTYPE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginType pt = PluginType.getNew(dc, interfaceName, jarPath);
			if (setAll)
			{
				pt.setName("Test plugin type");
				pt.setDescription("Added at "+new Date());
			}
			dc.saveItem(pt);
			dc.commit();
			id = pt.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(pt, false);
			write_item(0, pt);
			write("--Create plugin type OK");
		}
		catch (Throwable ex)
		{
			write("--Create plugin type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginType pt = PluginType.getById(dc, id);
			write_item(0, pt);
			write("--Load plugin type OK");
		}
		catch (Throwable ex)
		{
			write("--Load plugin type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<PluginType> l = PluginType.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List plugin types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List plugin types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginType pt = PluginType.getById(dc, id);
			dc.deleteItem(pt);
			dc.commit();
			write("--Delete plugin type OK");
		}
		catch (Throwable ex)
		{
			write("--Delete plugin type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tInterface\tJar path\tDescription");
			write("-- \t-- \t--------- \t---------\t--------\t-----------");
		}
	}
	
	static void write_item(int i, PluginType pt)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+pt.getId()+"\t"+pt.getName()+"\t"+
			pt.getInterfaceName()+"\t"+pt.getJarPath()+"\t"+pt.getDescription());
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
}
